local addonName, vars = ...
_G[addonName] = vars
local addon = vars

local rev = tonumber(("$Revision: 2 $"):match("%d+"))
local function msg(...)
    print("\124cff00ff00CalendarKeyboardFixer\124r: ",...)
end
local debug_enabled = true
local function debug(...)
  if debug_enabled then
    msg(...)
  end
end

--[[
  Here is one LUA stack for the problem we are fixing: (see Blizzard_Calendar/Blizzard_Calendar.lua)
 
  CalendarCreateEventDescriptionEdit:ClearFocus()
  CalendarCreateEventFrame_Update  ( CalendarCreateEventFrame.mode == "edit" ) 
  CalendarCreateEventFrame_OnEvent  ( event == "GUILD_ROSTER_UPDATE" )
      
  CalendarCreateEventFrame_OnLoad(self) => self:RegisterEvent("GUILD_ROSTER_UPDATE");
--]]        

local function highlight_dummy(self,start,stop)
  debug("highlight_dummy")
end

local function wrapped_update()
  debug("Called CalendarCreateEventFrame_Update")
  local focus = GetCurrentKeyBoardFocus()
  local cpos, spos
  local sf = CalendarCreateEventDescriptionScrollFrame
  local focusht
  -----------------------------------
  -- SAVE STATE
  -----------------------------------
  if focus then 
     if focus.GetCursorPosition then
       cpos = focus:GetCursorPosition();
     end
     if focus == CalendarCreateEventDescriptionEdit and sf and sf.GetVerticalScroll then
       spos = sf:GetVerticalScroll();
     elseif focus == CalendarCreateEventTitleEdit then
       -- 
     else -- unrecognized focus 
       focus = nil
     end
     if focus and focus.HighlightText then 
       -- Some OnEditFocusLost/Gained scripts change the highlighted text
       -- we apparently cannot query the EditBox highlighted text to save/restore,
       -- so instead we temporarily disable it entirely
       focusht = focus.HighlightText
       focus.HighlightText = highlight_dummy
     end
  end

  -----------------------------------
  -- CALL BLIZZARD UPDATE
  -----------------------------------
  addon.raw_update() 

  -----------------------------------
  -- RESTORE STATE
  -----------------------------------
  local nfocus = GetCurrentKeyBoardFocus()
  if focus and nfocus ~= focus then -- restore
    debug("Lost focus!! Fixing...")
    focus:SetFocus()
    if cpos and focus.SetCursorPosition then
      focus:SetCursorPosition(cpos)
    end
    if spos and sf and sf.SetVerticalScroll then sf:SetVerticalScroll(spos); end
    if focus == CalendarCreateEventTitleEdit then
      --focus:HighlightText(0, 0) -- undo auto-select all
    end
  end
  if focusht then
    focus.HighlightText = focusht
  end
end

local function UpdateHooks()
  if addon.hookdone then return end

  if not CalendarCreateEventFrame or 
     not CalendarCreateEventFrame_Update then 
     return 
  end
  addon.raw_update = CalendarCreateEventFrame_Update
  CalendarCreateEventFrame_Update = wrapped_update
  CalendarCreateEventFrame.update = wrapped_update

  addon.hookdone = true
  msg("Loaded rev "..rev)
end

local function frame_OnEvent(self,event,...)
 if event == "ADDON_LOADED" then
   UpdateHooks()
 end
end

addon.frame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
addon.frame:SetScript("OnEvent", frame_OnEvent)
addon.frame:RegisterEvent("ADDON_LOADED")

